$fn=50;

use<ido.scad>

INCH = 25.4;

spacing = 1 * INCH;
board_thickness = 5.5;
hole_dia = 0.9 *(3/16) * INCH;

wall = 2;
clearance = 2;
w=1.5;


module oval(d1, d2, h){
    difference(){
        scale([1, d2/d1, 1]) cylinder(d=d1+2*w, h=h);
        translate([0,0,w]) scale([1, d2/d1, 1]) cylinder(d=d1, h=h);
    }
}

module peg(){
    cylinder(d=hole_dia, h=board_thickness, $fn=50);
    translate([0, 0, board_thickness]) 
      sphere(d=hole_dia);
}

module hook(){
    peg();
    translate([board_thickness, 0, board_thickness])
    rotate([90,0,0])
      intersection() {
        rotate_extrude(convexity = 20, $fn=50) 
          translate([board_thickness, 0, 0])
            circle(r = hole_dia/2, $fn = 100);
        translate([-2*board_thickness, 0, -hole_dia/2])
          cube([2*board_thickness, 2*board_thickness, board_thickness]);
      }
}

module base(w, h){
    mirror([0, 0, 1]){
      rounded_cube([w*spacing+hole_dia+2*clearance, h*spacing+hole_dia+2*clearance, wall], 5, flat_z=true);
      for (j=[0:h])
          translate([w*spacing+hole_dia/2+clearance, j*spacing+hole_dia/2+clearance, 0])
            hook();
      for (j=[0:h])
          translate([0*spacing+hole_dia/2+clearance, j*spacing+hole_dia/2+clearance, 0])
            peg();
    }
}

module top_screwdriver(n) {
    d1 = 5;
    d2 = 13.5;
    for (i=[0:n-1])
      translate([0,(i+0.5)*spacing+hole_dia/2 + clearance,  15-1.5*w])
        rotate([90, 180, 90]) {
            translate([0,-2,0]) {
                tube(h=15, d_outer=13.5+2*w, d_inner=d1);
                translate([0, 0, 15])     difference(){
                  cylinder(d1=d2+2*w, d2=d2+2*w, h=5);
                  cylinder(h=5, d1=d1, d2=d2);
                }
                translate([0, 0, 20]) tube(d_inner=d2, d_outer=d2+2*w, h=6);
            }
            // arm
            translate([-wall, w+d1/2, 0]) cube([2*wall, (spacing-d1-2*w)/2, 15]);
        }
}

module top_superglue(){
    x = 38;
    y = 25;
   translate([0, spacing+hole_dia/2+clearance, y/2])
    rotate([0, 90, 0])
       oval(y, x, 15);
    
}

module top_measuring_tape(){
    translate([0, 60+2*wall-2.5, 30+2*wall])
    rotate([90, 180, 90])
    
    difference(){
     cube([60+2*wall, 30+2*wall, 30+clearance+wall]);
        translate([wall, wall, wall])
           cube([60, 23, 30+clearance]);
        translate([wall+16, wall+23, wall])
           cube([25, 27, 30+clearance]);
        translate([wall,wall,0])
           cube([5, 23, wall]);
        translate([30+wall, 0, 0])
          cylinder(h=30+clearance+wall, d1=20, d2=30);
    }
}


module top_ruller(){
    difference(){
      rounded_cube([spacing+hole_dia+2*clearance, spacing+hole_dia+2*clearance, 2+wall],
        5, flat_z=true);
      translate([0, hole_dia/2+clearance, 2]) 
        cube([spacing+hole_dia+2*clearance, spacing, wall]);
      translate([spacing/2+hole_dia/2+clearance,   spacing/2+hole_dia/2+clearance,   0])
        cylinder(h=2, d=10.5);
    }
}

module top_plaier(){
  difference(){
    translate([0, 0, -2]) rounded_cube([5, spacing+hole_dia+2*clearance, 12+2], r=2);
    translate([5.1, 11.5, 2]) rotate([0, -90, 0]) rounded_cube([8, 11, 5.2], r=2, flat_z = true);
  }
}

module top_flashlight(){
    d1 = 5;
    d2 = 28;
    d3 = 38;
    pole_h = d2 + (d3 - d2)/2;
    translate([spacing/2,12,0]) for (i=[0,1]){
        translate([0, i*(d2+d1), 0]) cylinder(d=d1, h=pole_h);
        translate([0, i*(d2+d1), pole_h]) sphere(d=d1+2);
    }
}

module top_flashlight2(){
    translate([0, spacing/2 + hole_dia/2 + clearance, 20])
    rotate([-90,0,-90])
    difference(){
        union(){
            cylinder(h=20, d=38);
            translate([0, 10, 10]) cube([20, 20, 20], center=true);
        }
        cylinder(h=5, d=28);
        translate([0,, 0,, 5]) cylinder(h=15, d1=28, d2=34);
       translate([-28/2, -28, 0]) cube([28, 28, 20]);
    }
}

module top_twisers(){
  difference(){
    translate([0, 0, -2]) rounded_cube([5, spacing+hole_dia+2*clearance, 15+2], r=2);
      for (i=[0:3])
        translate([5.1, 4.5+7*i, 1.25]) rotate([0, -90, 0]) rounded_cube([11, 4, 5.2], r=0.5, flat_z = true);
  }
}

module top_cylinder(dia, height){
    translate([0, spacing/2 + hole_dia/2 + clearance, dia/2])
      rotate([0, 90, 0]) 
        difference() {
            cylinder(d=dia+2*wall, h=height);
            translate([0, 0, wall]) cylinder(d=dia, h=height-wall);
        }
}

module top_tourch(){
    top_cylinder(30.5, INCH);
}

module top_gluestick(){
    top_cylinder(28.5, 1.3*INCH);
}

module top_ikea_measuring_tape(){
    translate([0, 62.5+2*wall-2.5, 25+2*wall])
      rotate([90, 180, 90])
        difference(){
         cube([62.5+2*wall, 25+2*wall, 30+clearance+wall]);
         translate([wall, wall, wall])
            cube([62.5, 25, 30+clearance]);
         //translate([wall+16, wall+23, wall])
         //   cube([25, 27, 30+clearance]);
         translate([wall,wall,0])
            #cube([5, 25, wall]);
         translate([30+wall, 0, 0])
           cylinder(h=30+clearance+wall, d1=20, d2=30);
        }
}

base (1, 2);
top_ikea_measuring_tape();